package gonganxitong

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/aaa"
	"shylinux.com/x/icebergs/base/ctx"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/tcp"
	"shylinux.com/x/icebergs/base/web"
	"shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/gonganxitong/model"
)

type Portal struct {
	ice.Hash
	Table
	user        user
	city        city
	event       event
	message     message
	recent      recent
	command     command
	service     service
	export      string `data:"true"`
	short       string `data:"index"`
	field       string `data:"time,icons,name,index,order,enable,init,type,role,view,command_uid"`
	create      string `name:"create index name icons"`
	list        string `name:"list place_uid index uid auto" role:"void"`
	placeCreate string `name:"placeCreate city_name* street_name* place_type*:select place_name* address*" icon:"bi bi-plus-square-dotted" role:"void"`
	placeRemove string `name:"placeRemove uid*" role:"void"`
	placeAuth   string `name:"placeAuth" role:"void"`
	placeSearch string `name:"placeSearch" icon:"bi bi-search" role:"void"`
	scanQRCode  string `name:"scanQRCode type text" icon:"bi bi-qr-code-scan" role:"void"`
}

func (s Portal) Init(m *ice.Message, arg ...string) {
	m.Design(s.List, "", kit.JoinWord(s.Keys(s.Place, model.UID), ctx.INDEX, model.UID, ice.AUTO))
	s.LoadTrans(m, arg...)
	s.Hash.Init(m, arg...)
}
func (s Portal) Exit(m *ice.Message, arg ...string) {
	mdb.HashSelectUpdate(m.Message, "", func(value ice.Map) {
		delete(value, model.COMMAND_UID)
		delete(value, mdb.TARGET)
		if value[mdb.ENABLE] == ice.TRUE {
			delete(value, mdb.ENABLE)
		}
		delete(value, mdb.ICONS)
		delete(value, mdb.NAME)
	})
	s.Hash.Exit(m, arg...)
}
func (s Portal) BeforeMigrate(m *ice.Message, arg ...string) {
	if m.Cmd(web.CODE_MYSQL_CLIENT).Length() == 0 {
		if m.Cmd(s.FindSpaceCmd(m, web.CODE_MYSQL_CLIENT)).Table(func(value ice.Maps) { m.Cmd(web.CODE_MYSQL_CLIENT, s.Create, aaa.SESS, "", kit.Simple(value)) }).Length() == 0 {
			m.Cmd(web.CODE_MYSQL_CLIENT, s.Create, aaa.SESS, "mysql", "driver", "mysql", "database", "mysql", tcp.HOST, "localhost", tcp.PORT, "10001", aaa.USERNAME, "root", aaa.PASSWORD, "root")
		}
	}
}
func (s Portal) AfterMigrate(m *ice.Message, arg ...string) {
	if name := kit.Select("", kit.Split(m.PrefixKey(), "."), -2); !m.Exists("src/" + name) {
		return
	}
	cmd := m.GetCommand()
	m.Cmdy(s.Prefix(m, s.service), s.Table.Update, ctx.INDEX, m.PrefixKey(), mdb.NAME, cmd.Help+" "+ice.Info.Title(), mdb.ICON, m.Resource(kit.Select(ice.Info.NodeIcon, cmd.Icon)))
}
func (s Portal) Inputs(m *ice.Message, arg ...string) {
	m.Cmdy(s.Place, m.ActionKey(), arg)
}
func (s Portal) Run(m *ice.Message, arg ...string) {
	s.DisplayBase(m, "common.js")
	m.Search(arg[0], func(key string, cmd *ice.Command) {
		sub, role := "", cmd.Role
		if len(arg) > 1 && arg[1] == ctx.ACTION {
			if action, ok := cmd.Actions[arg[2]]; ok {
				sub, role = arg[2], action.Role
			}
		} else if len(arg) > 1 {
			if action, ok := cmd.Actions[arg[1]]; ok {
				sub, role = arg[1], action.Role
			}
		}
		if kit.IsIn(role, "", aaa.VOID) {
			kit.If(!ctx.PodCmd(m.Message, arg) && aaa.Right(m.Message, arg), func() { m.Cmdy(arg) })
		} else {
			if !m.Cmdy(arg[0], s.CheckRole, role).IsErr() {
				m.Cmdy(arg)
			}
		}
		if sub != mdb.INPUTS {
			s.Place.RewriteAppend(m)
		}
	})
}
func (s Portal) List(m *ice.Message, arg ...string) {
	if m.Option(mdb.VIEW) == mdb.TABLE {
		s.Hash.List(m, arg...).PushAction(mdb.DETAIL, s.Hash.Remove).SortInt(mdb.ORDER)
		return
	}
	if len(arg) == 0 {
		USER_PLACE_ROLE := s.Keys(s.UserPlace, model.ROLE)
		m.Cmdy(m.PrefixKey(), s.PlaceList, m.Option(model.USER_UID)).Table(func(value ice.Maps) {
			switch UserPlaceRole(kit.Int(value[USER_PLACE_ROLE])) {
			case UserPlaceCreator, UserPlaceLandlord:
				if value[model.AUTH_STATUS] == "" {
					m.PushButton(s.PlaceAuth, s.PlaceRemove)
					break
				}
				fallthrough
			default:
				m.PushButton(s.PlaceRemove)
			}
		}).Action(s.PlaceCreate, s.ScanQRCode, s.PlaceSearch)
		kit.If(!m.IsErr() && m.Length() == 0, func() { m.EchoInfoButton("", s.PlaceCreate, s.ScanQRCode, s.PlaceSearch) })
	} else if len(arg) == 1 {
		if arg[0] != "" {
			msg := s.PlaceListOption(m, arg...)
			m.Option(model.PLACE_NAME, msg.Append(s.Keys(s.Place, model.NAME)))
			m.Option(model.STREET_NAME, msg.Append(s.Keys(s.Street, model.NAME)))
			s.AddRecent(msg, arg[0])
		}
		s.Hash.List(m).SortInt(mdb.ORDER)
	} else if len(arg) == 2 {
		s.PlaceListOption(m, arg...)
		m.Option(model.COMMAND_UID, s.Hash.List(m.Spawn(), arg[1]).Append(model.COMMAND_UID))
		m.Cmdy(ctx.COMMAND, arg[1]).Push(ctx.ARGS, arg[0])
	} else {
		m.FieldsSetDetail().Cmdy(arg[1], mdb.SELECT, model.UID, arg[2]).Action()
	}
	s.Place.RewriteAppend(m)
	s.DisplayBase(m, "").DisplayCSS("")
}
func (s Portal) PlaceListOption(m *ice.Message, arg ...string) *ice.Message {
	msg := m.Cmd(m.PrefixKey(), s.PlaceList, m.Option(model.USER_UID), arg[0])
	if msg.Length() == 0 {
		msg = m.Cmd(s.Place, s.Table.Select, model.UID, arg[0])
		msg.Push(s.Keys(s.UserPlace, model.ROLE), "0")
		msg.RenameAppend(model.NAME, s.Keys(s.Place, model.NAME), model.TYPE, s.Keys(s.Place, model.TYPE))
		s.SelectJoin(msg, s.Street, model.NAME, model.CITY_UID)
		s.SelectJoinCity(msg)
	}
	s.Place.RewriteAppend(msg)
	m.Options(msg.AppendSimple(model.AUTH_STATUS, s.Keys(s.Place, model.NAME), s.Keys(s.Place, model.TYPE), s.Keys(s.UserPlace, model.ROLE), s.Keys(s.Street, model.NAME), model.CITY_NAME))
	return msg
}
func (s Portal) PlaceCreate(m *ice.Message, arg ...string) {
	defer m.ToastProcess()()
	if s.city.FindOrCreateByName(m, arg...); m.IsErr() {
		return
	} else if s.Street.FindOrCreateByName(m, arg...); m.IsErr() {
		return
	}
	name := m.Option(s.Keys(s.Place, model.NAME))
	arg = kit.TransArgKeys(arg, s.Keys(s.Place, model.TYPE), model.TYPE)
	arg = kit.TransArgKeys(arg, s.Keys(s.Place, model.NAME), model.NAME)
	if m.Cmdy(s.Place, s.Create, arg[2:]).IsErr() {
		return
	}
	s.RecordEventWithName(m.Options(mdb.NAME, name, s.Keys(s.Place, model.UID), m.Result()), "")
	args := kit.Simple(m.OptionSimple(s.Keys(s.Place, model.UID), model.USER_UID), model.ROLE, UserPlaceCreator)
	m.SetResult().Cmdy(s.UserPlace, s.Create, args)
}
func (s Portal) PlaceRemove(m *ice.Message, arg ...string) {
	defer m.ToastProcess()()
	m.Cmdy(s.UserPlace, s.Delete, m.OptionSimple(model.USER_UID, model.UID))
}
func (s Portal) PlaceList(m *ice.Message, arg ...string) *ice.Message {
	s.Tables(m, s.Place).FieldsWithCreatedAT(m, s.UserPlace, model.AUTH_UID, s.Key(s.Place, model.INIT),
		s.Keys(s.Place, model.NAME), s.Keys(s.Place, model.TYPE), s.Keys(s.UserPlace, model.ROLE),
		s.Keys(s.Place, model.UID), s.Keys(s.Street, model.UID),
	)
	if len(arg) == 1 {
		m.Cmdy(s.UserPlace, s.Table.Select, model.USER_UID, arg[0])
	} else if len(arg) == 2 {
		m.FieldsSetDetail().Cmdy(s.UserPlace, s.Table.Select, model.USER_UID, arg[0], s.Keys(s.Place, model.UID), arg[1])
	} else {
		return m
	}
	s.SelectJoin(m, s.Street, model.NAME, model.CITY_UID)
	s.SelectJoinCity(m)
	s.SelectJoinAuth(m)
	return m
}
func (s Portal) PlaceAuth(m *ice.Message, arg ...string) {
	if m.Option(model.AUTH_UID) == "" {
		place_uid := m.Option(s.Keys(s.Place, model.UID))
		place_name := m.Option(s.Keys(s.Place, model.NAME))
		m.Option(model.COMPANY_NAME, m.Option(s.Keys(s.Street, model.NAME)))
		msg := s.SpaceCmd(m, "20240724-community", "web.team.renzhengshouquan.portal", "placeCreate", m.OptionSimple(model.CITY_NAME, model.COMPANY_NAME),
			model.AUTH_TYPE, ice.AUTO, model.AUTH_NAME, place_name, model.FROM_UID, ice.AUTO)
		if msg.IsErr() {
			m.Copy(msg)
			return
		}
		m.Cmd(s.Place, s.UpdateAuth, msg.Option(model.AUTH_UID), model.UID, place_uid)
		s.RecordEventWithName(m.Options(model.NAME, place_name, model.UID, place_uid), "")
	} else {
		m.ProcessHold()
	}
}
func (s Portal) PlaceSearch(m *ice.Message, arg ...string) {
	m.ProcessRewrite(model.INDEX, s.Prefix(m, search{}))
}
func (s Portal) ScanQRCode(m *ice.Message, arg ...string) {
	defer m.ToastProcess()()
	if m.Option(mdb.TYPE) == mdb.TEXT {
		m.Cmdy(s.UserPlace, s.Create, s.Key(s.Place, model.UID), m.Option(mdb.TEXT), m.OptionSimple(model.USER_UID), model.ROLE, UserPlaceVisitor)
	} else if m.Option(mdb.TYPE) == web.LINK {
		args := m.ParseURL(m.Option(mdb.TEXT))
		if len(args) > 1 && args[1] == s.Prefix(m, apply{}) {
			m.Cmdy(s.UserPlace, s.Create, s.Keys(s.Place, model.UID), args[0], m.OptionSimple(model.USER_UID), model.ROLE, UserPlaceVisitor)
		}
	}
}
func (s Portal) UserPlaceRole(m *ice.Message, arg ...string) {
	m.Echo(m.Cmd(s.UserPlace, s.Table.Select, m.OptionSimple(s.Keys(s.Place, model.UID), model.USER_UID)).Append(model.ROLE))
}
func (s Portal) UserPlaceCmd(m *ice.Message, arg ...string) {
	m.Cmdy(s.UserPlace, arg)
}
func (s Portal) PlaceCmd(m *ice.Message, arg ...string) {
	m.Cmdy(s.Place, arg)
}
func (s Portal) RecordEvent(m *ice.Message, arg ...string) {
	s.event.Record(m.Spawn(kit.Dict(model.PLACE_UID, m.Option(s.Keys(s.Place, model.UID)))), arg[0], arg[1:]...)
}
func (s Portal) AddRecent(m *ice.Message, arg ...string) {
	m.Options(m.ConfigSimple(model.SERVICE_UID))
	if kit.IsIn(m.Append(model.AUTH_STATUS), "2", "issued") {
		m.Cmd(s.Prefix(m, s.recent), s.Create, s.GetCommands(m, model.SERVICE_UID, arg...), model.AUTH_STATUS, "2")
	} else {
		m.Cmd(s.Prefix(m, s.recent), s.Create, s.GetCommands(m, model.SERVICE_UID, arg...))
	}
}
func (s Portal) ValueCreate(m *ice.Message, arg ...string) {
	s.Table.Create(m, kit.Simple(arg, m.OptionSimple(model.USER_UID, s.Keys(s.Place, model.UID)))...)
}
func (s Portal) ValueRemove(m *ice.Message, arg ...string) {
	s.Table.Delete(m, m.OptionSimple(s.Keys(s.Place, model.UID), model.UID)...)
}
func (s Portal) ValueModify(m *ice.Message, arg ...string) {
	s.Table.Update(m, kit.Dict(arg), m.OptionSimple(s.Keys(s.Place, model.UID), model.UID)...)
}
func (s Portal) ValueList(m *ice.Message, arg ...string) {
	s.TablesWithRole(m, arg, s.UserPlace, s.Place, m.Option("target"), m.Optionv(mdb.FIELDS).([]ice.Any)...)
}

func init() { ice.TeamCtxCmd(Portal{Table: newTable()}) }

func (s Portal) UpdateCommand(m *ice.Message, arg ...string) {
	m.Config(model.SERVICE_UID, arg[0])
	s.Hash.Select(m.Spawn()).Table(func(value ice.Maps) {
		uid := m.Cmdx(s.command, s.Table.Update, model.SERVICE_UID, arg[0], mdb.ICON, value[mdb.ICONS], model.NAME, value[model.NAME], model.INDEX, value[model.INDEX])
		s.Hash.Modify(m, ctx.INDEX, value[ctx.INDEX], model.COMMAND_UID, uid)
	})
}
func (s Portal) SelectCommand(m *ice.Message, arg ...string) {
	s.Hash.Select(m, arg...)
}
func (s Portal) Create(m *ice.Message, arg ...string) {
	args := []string{ctx.INDEX, m.Option(ctx.INDEX)}
	msg := s.Hash.Select(m.Spawn(), m.Option(ctx.INDEX))
	kit.If(msg.Length() > 0, func() { args = append(args, mdb.TIME, msg.Append(mdb.TIME)) })
	m.Search(m.Option(ctx.INDEX), func(key string, cmd *ice.Command) {
		args = append(args, mdb.NAME, cmd.Help, mdb.ICONS, cmd.Icon)
		kit.For([]string{mdb.ORDER, mdb.TYPE, aaa.ROLE, mdb.VIEW}, func(key string) {
			value := m.Conf(m.Option(ctx.INDEX), kit.Keym(key))
			args = append(args, key, kit.Select(value, msg.Append(key)))
		})
	})
	s.Hash.Create(m, args...)
}
func (s Portal) Show(m *ice.Message, arg ...string) {
	m.Cmd(s.Prefix(m, s), s.Create, ctx.INDEX, m.PrefixKey())
}
func (s Portal) Link(m *ice.Message, arg ...string) string {
	return m.MergePodCmd("", s.Prefix(m, s)) + "#" + kit.TrimSuffix(kit.Join([]string{arg[0], kit.Select(s.Prefix(m, apply{}), arg, 1), kit.Select("", arg, 2)}, ":"), ":")
}

func init() { ice.TeamCtxCmd(Portal{Table: newTable()}) }
