package renzhengshouquan

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/renzhengshouquan/model"
)

type auth struct {
	Table
	certList   string `name:"certList" role:"leader,worker"`
	memberList string `name:"memberList" role:"leader,worker"`
	issue      string `name:"issue" role:"leader,worker"`
	revoke     string `name:"revoke" role:"leader,worker"`
}

func (s auth) List(m *ice.Message, arg ...string) {
	if len(arg) == 1 {
		s.Select(m, model.FROM_UID, arg[0])
	} else if len(arg) == 2 {
		s.SelectDetail(m, model.FROM_UID, arg[0], model.UID, arg[1]).Action(s.CertList, s.MemberList)
	}
	m.Table(func(value ice.Maps) {
		switch AuthStatus(kit.Int(value[model.STATUS])) {
		case AuthCreate:
			m.PushButton(s.Issue)
		case AuthIssued:
			m.PushButton(s.Revoke)
		case AuthRevoked:
			m.PushButton(s.Issue)
		default:
			m.PushButton()
		}
	}).RenameAppend(model.TYPE, model.AUTH_TYPE, model.STATUS, model.AUTH_STATUS).Display("")
}
func (s auth) CertList(m *ice.Message, arg ...string) {
	m.Cmdy(cert{}, arg[0]).Action()
	kit.If(m.Length() == 0, func() { m.Echo("没有上传证件") })
}
func (s auth) MemberList(m *ice.Message, arg ...string) {
	m.Cmdy(m.Prefix("member"), arg[0])
}
func (s auth) Issue(m *ice.Message, arg ...string) {
	msg := m.Cmd(userAuth{}, m.Option(model.USER_UID), m.Option(model.AUTH_UID))
	if !m.IsTech() || AuthType(kit.Int(msg.Append(model.AUTH_TYPE))) != AuthRoot {
		if m.WarnNotValid(AuthStatus(kit.Int(msg.Append(model.AUTH_STATUS))) != AuthIssued) {
			return
		}
	}
	s.Update(m, kit.Dict(model.STATUS, AuthIssued), model.FROM_UID, m.Option(model.AUTH_UID), model.UID, m.Option(model.UID))
	switch s.Select(m, model.UID, m.Option(model.UID)); AuthType(kit.Int(m.Append(model.TYPE))) {
	case AuthCity:
		m.Cmd("web.team.gonganxitong.city", s.UpdateAuth, m.Option(model.UID), model.NAME, m.Append(model.NAME))
	case AuthPersonal:
		m.Cmd("web.team.gonganxitong.user", s.UpdateAuth, m.Option(model.UID), model.UID, m.Append(model.CREATOR))
	case AuthCompany:
		m.Cmd("web.team.guanlixitong.company", s.UpdateAuth, m.Option(model.UID), model.NAME, m.Append(model.NAME))
	}
}
func (s auth) Revoke(m *ice.Message, arg ...string) {
	s.Update(m, kit.Dict(model.STATUS, AuthRevoked), model.FROM_UID, m.Option(model.AUTH_UID), model.UID, m.Option(model.UID))
}

func init() { ice.TeamCtxCmd(auth{}) }

type AuthType int

const (
	AuthRoot AuthType = iota
	AuthCity
	AuthCompany
	AuthService
	AuthPersonal
)

var AuthTypeList = map[AuthType]string{
	AuthRoot:     "root",
	AuthCity:     "city",
	AuthCompany:  "company",
	AuthService:  "service",
	AuthPersonal: "personal",
}

func (s AuthType) String() string { return AuthTypeList[s] }

type AuthStatus int

const (
	AuthCreate AuthStatus = iota
	AuthRequest
	AuthIssued
	AuthRevoked
)

var AuthStatusList = map[AuthStatus]string{
	AuthCreate:  "create",
	AuthRequest: "request",
	AuthIssued:  "issued",
	AuthRevoked: "revoked",
}

func (s AuthStatus) String() string { return AuthStatusList[s] }
