package renzhengshouquan

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/renzhengshouquan/model"
	"shylinux.com/x/enterprise/src/guanlixitong"
)

type Table struct {
	guanlixitong.Table
	list string `name:"list auth_uid uid auto" role:"void"`
}

func (s Table) Inputs(m *ice.Message, arg ...string) {
	switch arg[0] {
	case model.USER_AUTH_ROLE:
		s.InputsListRole(m, UserAuthRoleList, arg...)
	case model.AUTH_TYPE:
		if m.IsTech() {
			s.InputsList(m, AuthTypeList, arg...)
		} else {
			for i, v := range AuthTypeList {
				kit.If(i > 0, func() { m.Push(arg[0], i).Push(model.NAME, v) })
			}
			m.SortInt(arg[0]).DisplayInputKeyNameIconTitle()
		}
	case model.AUTH_STATUS:
		s.InputsList(m, AuthStatusList, arg...)
	case model.FROM_UID:
		switch AuthType(kit.Int(m.Option(model.AUTH_TYPE))) {
		case AuthRoot:
			m.Push(model.UID, "root").Push(model.NAME, "root")
		case AuthCity:
			m.Cmdy(auth{}, s.Select, model.TYPE, AuthRoot)
		case AuthCompany, AuthPersonal:
			m.Cmdy(auth{}, s.Select, model.TYPE, AuthCity)
		default:
			msg := m.Cmd(auth{}, s.Select, model.TYPE, AuthCity, model.NAME, m.Option(model.CITY_NAME))
			m.Cmdy(auth{}, s.Select, model.TYPE, AuthCompany, model.FROM_UID, msg.Append(model.UID))
		}
		m.Cut(model.UID, model.NAME).RenameAppend(model.UID, model.FROM_UID)
		m.DisplayInputKeyNameIconTitle()
	default:
		s.Table.Inputs(m, arg...)
	}
}
func (s Table) RewriteAppend(m *ice.Message, arg ...string) *ice.Message {
	m.RewriteAppend(func(value, key string, index int) string {
		switch key {
		case model.USER_AUTH_ROLE:
			value = UserAuthRole(kit.Int(value)).String()
		case model.AUTH_TYPE:
			value = AuthType(kit.Int(value)).String()
		case model.AUTH_STATUS:
			value = AuthStatus(kit.Int(value)).String()
		}
		return value
	})
	return s.Table.RewriteAppend(m)
}

type Tables struct{ Table }

func (s Tables) BeforeMigrate(m *ice.Message, arg ...string) {}

type Value struct {
	Table
	fields string `data:"title,content"`
	create string `name:"create title* content*" role:"leader"`
	modify string `name:"modify title* content*" role:"leader"`
	remove string `name:"remove" role:"leader"`
}

func (s Value) Create(m *ice.Message, arg ...string) { s.ValueCreate(m, arg...) }
func (s Value) Modify(m *ice.Message, arg ...string) { s.ValueModify(m, arg...) }
func (s Value) Remove(m *ice.Message, arg ...string) { s.ValueRemove(m, arg...) }
func (s Value) List(m *ice.Message, arg ...string)   { s.ValueList(m, arg, model.TITLE, model.CONTENT) }
